@@ -20,14 +20,8 @@
 #include "GLFW/glfw3.h"
 
 #define SK_GL
-#include "include/gpu/GrBackendSurface.h"
-#include "include/gpu/GrDirectContext.h"
-#include "include/gpu/gl/GrGLInterface.h"
-#include "include/core/SkCanvas.h"
-#include "include/core/SkSurface.h"
 #include "include/core/SkDocument.h"
 #include "include/docs/SkPDFDocument.h"
-#include "include/core/SkStream.h"
 #include "include/core/SkPictureRecorder.h"
 #include "include/core/SkPicture.h"
 #include "include/svg/SkSVGCanvas.h"
@@ -36,6 +30,8 @@
     #include <emscripten.h>
 #endif
 
+// note to developer - update version in workflows/main.yml, menu.cpp, term_out.cpp, and deps/gw.desktop, and installers .md in docs (post release)
+const char GW_VERSION [7] = "1.1.2";
 // skia context has to be managed from global space to work
 GrDirectContext *sContext = nullptr;
 SkSurface *sSurface = nullptr;
@@ -50,17 +46,14 @@ void print_banner() {
  "/   \\  __\\   \\/\\/   /\n"
  "\\    \\_\\  \\        / \n"
  " \\______  /\\__/\\  /  \n"
- "        \\/      \\/  " << std::endl;
+ "        \\/      \\/  " << GW_VERSION << std::endl;
 #else
     std::cout << "\n"
                  "█▀▀ █ █ █\n"
-                 "█▄█ ▀▄▀▄▀" << std::endl;
+                 "█▄█ ▀▄▀▄▀ v" << GW_VERSION << std::endl;
 #endif
 }
 
-// note to developer - update version in workflows/main.yml, menu.cpp, term_out.cpp, and deps/gw.desktop, and installers .md in docs
-const char GW_VERSION [7] = "1.1.1";
-
 
 bool str_is_number(const std::string &s) {
     return !s.empty() && std::all_of(s.begin(), s.end(), ::isdigit);
@@ -88,6 +81,127 @@ void drawImageCommands(Manager::GwPlot &p, SkCanvas *canvas, std::vector<std::st
     }
 }
 
+
+void setupRenderTarget(int fb_width, int fb_height) {
+
+    sk_sp<const GrGLInterface> interface = GrGLMakeNativeInterface();
+
+#ifndef OLD_SKIA
+    if (!interface || !interface->validate()) {
+        std::cerr << "Error: skia GrGLInterface was not valid" << std::endl;
+        if (!interface) {
+            std::cerr << "    GrGLMakeNativeInterface() returned nullptr" << std::endl;
+            std::cerr << "    GrGLInterface probably missing some GL functions" << std::endl;
+        } else {
+            std::cerr << "    fStandard was " << interface->fStandard << std::endl;
+        }
+        std::cerr << "GL error code: " << glGetError() << std::endl;
+        std::exit(-1);
+    }
+
+    //
+    sContext = GrDirectContexts::MakeGL(interface).release();
+    if (!sContext) {
+        std::cerr << "Error: could not create skia context using MakeGL\n";
+        std::exit(-1);
+    }
+
+    GrGLFramebufferInfo framebufferInfo;
+    framebufferInfo.fFBOID = 0;
+
+    constexpr int fbFormats[2] = {GL_RGBA8, GL_RGB8};  // GL_SRGB8_ALPHA8
+    constexpr SkColorType colorTypes[2] = {kRGBA_8888_SkColorType, kRGB_888x_SkColorType};
+    int valid = false;
+    for (int i=0; i < 2; ++i) {
+
+        framebufferInfo.fFormat = fbFormats[i];
+
+        auto backendRenderTarget = GrBackendRenderTargets::MakeGL(
+                fb_width,
+                fb_height,
+                0,        // sampleCnt
+                0,        // stencilBits
+                framebufferInfo
+        );
+
+        if (!backendRenderTarget.isValid()) {
+            std::cerr << "ERROR: backendRenderTarget was invalid" << std::endl;
+            glfwTerminate();
+            std::exit(-1);
+        }
+
+        // Now create the surface
+        SkSurfaceProps props(0, kUnknown_SkPixelGeometry);
+        sSurface = SkSurfaces::WrapBackendRenderTarget(
+                sContext,
+                backendRenderTarget,
+                kBottomLeft_GrSurfaceOrigin,
+                colorTypes[i],
+                nullptr,
+                &props
+        ).release();
+
+        if (!sSurface) {
+            std::stringstream sstream;
+            sstream << std::hex << fbFormats[i];
+            std::string result = sstream.str();
+            std::cerr << "ERROR: sSurface could not be initialized (nullptr). The frame buffer format was 0x" << result << std::endl;
+            continue;
+        }
+        valid = true;
+        break;
+    }
+    if (!valid) {
+        std::cerr << "ERROR: could not create a valid frame buffer\n";
+        std::exit(-1);
+    }
+#else
+    sContext = GrDirectContext::MakeGL(interface).release();
+    if (!sContext) {
+        std::cerr << "Error: could not create skia-m93 context using MakeGL\n";
+        std::exit(-1);
+    }
+
+    GrGLFramebufferInfo framebufferInfo;
+    framebufferInfo.fFBOID = 0;
+
+    constexpr int fbFormats[2] = {GL_RGBA8, GL_RGB8};  // GL_SRGB8_ALPHA8
+    constexpr SkColorType colorTypes[2] = {kRGBA_8888_SkColorType, kRGB_888x_SkColorType};
+    int valid = false;
+    for (int i=0; i < 2; ++i) {
+        framebufferInfo.fFormat = fbFormats[i];  // GL_SRGB8_ALPHA8; //
+        GrBackendRenderTarget backendRenderTarget(fb_width, fb_height, 0, 0, framebufferInfo);
+        if (!backendRenderTarget.isValid()) {
+            std::cerr << "ERROR: backendRenderTarget was invalid" << std::endl;
+            glfwTerminate();
+            std::exit(-1);
+        }
+        sSurface = SkSurface::MakeFromBackendRenderTarget(sContext,
+                                                          backendRenderTarget,
+                                                          kBottomLeft_GrSurfaceOrigin,
+                                                          colorTypes[i], //kRGBA_8888_SkColorType,
+                                                          nullptr,
+                                                          nullptr).release();
+        if (!sSurface) {
+            std::stringstream sstream;
+            sstream << std::hex << fbFormats[i];
+            std::string result = sstream.str();
+            std::cerr << "ERROR: sSurface could not be initialized (nullptr). The frame buffer format was 0x" << result << std::endl;
+            continue;
+        }
+        valid = true;
+        break;
+    }
+    if (!valid) {
+        std::cerr << "ERROR: could not create a valid frame buffer\n";
+        std::exit(-1);
+    }
+#endif
+}
+
+
+
+
 int main(int argc, char *argv[]) {
 
 //    std::chrono::high_resolution_clock::time_point timer_point = std::chrono::high_resolution_clock::now();
@@ -605,11 +719,19 @@ int main(int argc, char *argv[]) {
         plotter.init(iopts.dimensions.x, iopts.dimensions.y);
         int fb_height, fb_width;
         glfwGetFramebufferSize(plotter.window, &fb_width, &fb_height);
+
+        //////
+
         plotter.setGlfwFrameBufferSize();
 
+//        setupRenderTarget(fb_height, fb_height);
+
+
         sk_sp<const GrGLInterface> interface = GrGLMakeNativeInterface();
+
+#ifndef OLD_SKIA
         if (!interface || !interface->validate()) {
-		    std::cerr << "Error: skia GrGLInterface was not valid" << std::endl;
+            std::cerr << "Error: skia GrGLInterface was not valid" << std::endl;
             if (!interface) {
                 std::cerr << "    GrGLMakeNativeInterface() returned nullptr" << std::endl;
                 std::cerr << "    GrGLInterface probably missing some GL functions" << std::endl;
@@ -620,7 +742,8 @@ int main(int argc, char *argv[]) {
             std::exit(-1);
         }
 
-        sContext = GrDirectContext::MakeGL(interface).release();
+        //
+        sContext = GrDirectContexts::MakeGL(interface).release();
         if (!sContext) {
             std::cerr << "Error: could not create skia context using MakeGL\n";
             std::exit(-1);
@@ -629,6 +752,67 @@ int main(int argc, char *argv[]) {
         GrGLFramebufferInfo framebufferInfo;
         framebufferInfo.fFBOID = 0;
 
+        constexpr int fbFormats[2] = {GL_RGBA8, GL_RGB8};  // GL_SRGB8_ALPHA8
+        constexpr SkColorType colorTypes[2] = {kRGBA_8888_SkColorType, kRGB_888x_SkColorType};
+        int valid = false;
+        for (int i=0; i < 2; ++i) {
+
+            framebufferInfo.fFormat = fbFormats[i];
+
+            auto backendRenderTarget = GrBackendRenderTargets::MakeGL(
+                    fb_width,
+                    fb_height,
+                    0,        // sampleCnt
+                    0,        // stencilBits
+                    framebufferInfo
+            );
+
+            if (!backendRenderTarget.isValid()) {
+                std::cerr << "ERROR: backendRenderTarget was invalid" << std::endl;
+                glfwTerminate();
+                std::exit(-1);
+            }
+
+            // Now create the surface
+            SkSurfaceProps props(0, kUnknown_SkPixelGeometry);
+            sSurface = SkSurfaces::WrapBackendRenderTarget(
+                    sContext,
+                    backendRenderTarget,
+                    kBottomLeft_GrSurfaceOrigin,
+                    colorTypes[i],
+                    nullptr,
+                    &props
+            ).release();
+
+            if (!sSurface) {
+                std::stringstream sstream;
+                sstream << std::hex << fbFormats[i];
+                std::string result = sstream.str();
+                std::cerr << "ERROR: sSurface could not be initialized (nullptr). The frame buffer format was 0x" << result << std::endl;
+                continue;
+            }
+            valid = true;
+            break;
+        }
+        if (!valid) {
+            std::cerr << "ERROR: could not create a valid frame buffer\n";
+            std::exit(-1);
+        }
+
+        auto imageInfo = SkImageInfo::MakeN32Premul(
+                iopts.dimensions.x * plotter.monitorScale,
+                iopts.dimensions.y * plotter.monitorScale);
+        sk_sp<SkSurface> rasterSurface = SkSurfaces::Raster(imageInfo);
+#else
+        sContext = GrDirectContext::MakeGL(interface).release();
+        if (!sContext) {
+            std::cerr << "Error: could not create skia-m93 context using MakeGL\n";
+            std::exit(-1);
+        }
+
+        GrGLFramebufferInfo framebufferInfo;
+        framebufferInfo.fFBOID = 0;
+
         constexpr int fbFormats[2] = {GL_RGBA8, GL_RGB8};  // GL_SRGB8_ALPHA8
         constexpr SkColorType colorTypes[2] = {kRGBA_8888_SkColorType, kRGB_888x_SkColorType};
         int valid = false;
@@ -660,10 +844,10 @@ int main(int argc, char *argv[]) {
             std::cerr << "ERROR: could not create a valid frame buffer\n";
             std::exit(-1);
         }
-
-        // initialize drawing surface
         sk_sp<SkSurface> rasterSurface = SkSurface::MakeRasterN32Premul(iopts.dimensions.x * plotter.monitorScale,
                                                                         iopts.dimensions.y * plotter.monitorScale);
+#endif
+
         plotter.rasterCanvas = rasterSurface->getCanvas();
         plotter.rasterSurfacePtr = &rasterSurface;
 
@@ -812,7 +996,7 @@ int main(int argc, char *argv[]) {
                     std::unique_ptr<SkCanvas> svgCanvas = SkSVGCanvas::Make(SkRect::MakeWH(iopts.dimensions.x, iopts.dimensions.y), &out);
                     if (svgCanvas) {
                         picture->playback(svgCanvas.get());
-                        svgCanvas->flush();
+//                        svgCanvas->flush();
                     };
                 }
 
@@ -821,8 +1005,18 @@ int main(int argc, char *argv[]) {
                 // Plot a png image, either of target region or whole chromosome
                 sk_sp<SkImage> img;
                 if (!plotter.regions.empty()) {  // plot target regions
+
+
+
+#ifndef OLD_SKIA
+                    auto imageInfo = SkImageInfo::MakeN32Premul(
+                            iopts.dimensions.x * plotter.monitorScale,
+                            iopts.dimensions.y * plotter.monitorScale);
+                    sk_sp<SkSurface> rasterSurface = SkSurfaces::Raster(imageInfo);
+#else
                     sk_sp<SkSurface> rasterSurface = SkSurface::MakeRasterN32Premul(iopts.dimensions.x,
                                                                                     iopts.dimensions.y);
+#endif
                     SkCanvas *canvas = rasterSurface->getCanvas();
                     drawImageCommands(plotter, canvas, extra_commands);
 
@@ -934,7 +1128,7 @@ int main(int argc, char *argv[]) {
                     }
                 }
             }
-
+        // Save images of all variants to a folder .png format
         } else if (program.is_used("--variants") && !program.is_used("--out-vcf")) {
             if (outdir.empty()) {
                 std::cerr << "Error: please provide an output directory using --outdir\n";
@@ -981,6 +1175,7 @@ int main(int argc, char *argv[]) {
                     auto *m = new Manager::GwPlot(genome, bam_paths, iopts, regions, tracks);
                     m->opts.theme.setAlphas();
                     m->setImageSize(iopts.dimensions.x, iopts.dimensions.y);
+                    m->makeRasterSurface();
                     m->opts.threads = 1;
                     for (auto &s: filters) {
                         m->addFilter(s);
@@ -1036,7 +1231,7 @@ int main(int argc, char *argv[]) {
                                             block += 1;
                                             mtx.unlock();
                                             Manager::GwPlot *plt = managers[this_block];
-                                            plt->makeRasterSurface();
+
 //                                            sk_sp<SkSurface> rasterSurface = SkSurface::MakeRasterN32Premul(iopts.dimensions.x, iopts.dimensions.y);
 //                                            SkCanvas *canvas = rasterSurface->getCanvas();
                                             for (int i = a; i < b; ++i) {
@@ -1090,8 +1285,6 @@ int main(int argc, char *argv[]) {
     }
     if (!iopts.no_show) {
         std::cout << "\nGw finished\n";
-    } else {
-        std::cout << "Gw finished\n";
     }
     return 0;
 };
