@@ -20,9 +20,6 @@
 
 #include <GLFW/glfw3.h>
 #define SK_GL
-#include "include/gpu/GrBackendSurface.h"
-#include "include/gpu/GrDirectContext.h"
-#include "include/gpu/gl/GrGLInterface.h"
 #include "include/core/SkCanvas.h"
 #include "include/core/SkSamplingOptions.h"
 #include "include/core/SkSurface.h"
@@ -38,6 +35,7 @@
 #include "themes.h"
 #include "utils.h"
 #include "menu.h"
+#include "parser.h"
 
 
 namespace Menu {
@@ -284,7 +282,7 @@ namespace Menu {
 
         if (opts.control_level.empty()) {
             if (opts.menu_table == Themes::MenuTable::MAIN) {
-                tip = opts.ini_path + "  v1.1.1";
+                tip = opts.ini_path + "  v1.1.2";
             }
             else if (opts.menu_table == Themes::MenuTable::GENOMES) { tip = "Use ENTER key to select genome, or RIGHT_ARROW key to edit path"; }
             else if (opts.menu_table == Themes::MenuTable::SHIFT_KEYMAP) { tip = "Change characters selected when using shift+key"; }
@@ -843,9 +841,10 @@ namespace Menu {
         if (Utils::startsWith(new_opt.value, "http") || Utils::startsWith(new_opt.value, "ftp")) {
             opts.myIni[new_opt.table][new_opt.name] = new_opt.value;
         } else {
-            std::filesystem::path path = new_opt.value;
+            std::string new_opt_path = Parse::tilde_to_home(new_opt.value);
+            std::filesystem::path path = new_opt_path;
             if (std::filesystem::exists(path)) {
-                opts.myIni[new_opt.table][new_opt.name] = new_opt.value;
+                opts.myIni[new_opt.table][new_opt.name] = new_opt_path;
             } else {
                 std::cerr << termcolor::red << "Error:" << termcolor::reset << " local path does not exist: " << path << std::endl;
             }
