@@ -17,13 +17,8 @@
 
 #include <GLFW/glfw3.h>
 #define SK_GL
-#include "include/gpu/GrBackendSurface.h"
-#include "include/gpu/GrDirectContext.h"
-#include "include/gpu/gl/GrGLInterface.h"
 #include "include/core/SkCanvas.h"
-#include "include/core/SkSamplingOptions.h"
 #include "include/core/SkSurface.h"
-#include "include/core/SkDocument.h"
 
 #include "ankerl_unordered_dense.h"
 #include "drawing.h"
@@ -44,14 +39,14 @@ namespace Manager {
 
     void HiddenWindow::init(int width, int height) {
         if (!glfwInit()) {
-            std::cerr << "ERROR: could not initialize GLFW3" << std::endl;
+            std::cerr << "Error: could not initialize GLFW3" << std::endl;
             std::exit(-1);
         }
         glfwWindowHint(GLFW_STENCIL_BITS, 8);
         glfwWindowHint(GLFW_VISIBLE, GLFW_FALSE);
         window = glfwCreateWindow(width, height, "GW", NULL, NULL);
         if (!window) {
-            std::cerr << "ERROR: could not create back window with GLFW3" << std::endl;
+            std::cerr << "Error: could not create back window with GLFW3" << std::endl;
             glfwTerminate();
             std::exit(-1);
         }
@@ -149,12 +144,9 @@ namespace Manager {
     }
 
     GwPlot::~GwPlot() {
-        if (window != nullptr && !drawToBackWindow) {
+        if (window != nullptr) {
             glfwDestroyWindow(window);
         }
-        if (backWindow != nullptr && drawToBackWindow) {
-            glfwDestroyWindow(backWindow);
-        }
         glfwTerminate();
         for (auto &bm: bams) {
             hts_close(bm);
@@ -175,6 +167,18 @@ namespace Manager {
     }
 
     int GwPlot::makeRasterSurface() {
+#ifndef OLD_SKIA
+        SkImageInfo imageInfo = SkImageInfo::MakeN32Premul(opts.dimensions.x * monitorScale,
+                                                           opts.dimensions.y * monitorScale);
+        size_t rowBytes = imageInfo.minRowBytes();
+        size_t size = imageInfo.computeByteSize(rowBytes);
+        this->pixelMemory.resize(size);
+
+        sk_sp<SkSurface> rasterSurface = SkSurfaces::WrapPixels(
+                imageInfo,
+                &pixelMemory[0],
+                rowBytes);
+#else
         SkImageInfo info = SkImageInfo::MakeN32Premul(opts.dimensions.x * monitorScale,
                                                       opts.dimensions.y * monitorScale);
         size_t rowBytes = info.minRowBytes();
@@ -182,12 +186,13 @@ namespace Manager {
         this->pixelMemory.resize(size);
         this->rasterSurface = SkSurface::MakeRasterDirect(
                 info, &pixelMemory[0], rowBytes);
+#endif
         rasterCanvas = rasterSurface->getCanvas();
         rasterSurfacePtr = &rasterSurface;
         if (pixelMemory.empty()) {
             std::cerr << "Error: pixelMemory empty, makeRasterSurface failed\n";
         }
-        setGlfwFrameBufferSize();
+
         return pixelMemory.size();
     }
 
@@ -198,32 +203,34 @@ namespace Manager {
 
         int major_v = -1;
         int minor_v = -1;
+        char *v_major = getenv("GLFW_CONTEXT_VERSION_MAJOR");
+        if (v_major) {
+            major_v = std::stoi(v_major);
+        }
+        char *v_minor = getenv("GLFW_CONTEXT_VERSION_MINOR");
+        if (v_minor) {
+            minor_v = std::stoi(v_minor);
+        }
 
-        char *val = getenv("GW_DEBUG");
-        bool debug = (val != nullptr);
+        char *val = getenv("GW_USE_GL");
+        bool use_gl = (val != nullptr && *val != '0');
+        val = getenv("GW_DEBUG");
+        bool debug = (val != nullptr && *val != '0');
         if (debug) {
-
-            const char* env_vars[] = {"DISPLAY",
+            std::cerr << "GW_USE_GL=" << use_gl << std::endl;
+            const char *env_vars[] = {"DISPLAY",
                                       "LIBGL_DEBUG", "LIBGL_ALWAYS_INDIRECT", "LIBGL_ALWAYS_SOFTWARE",
                                       "MESA_DEBUG", "MESA_GL_VERSION_OVERRIDE", "GLX_DEBUG",
                                       "GALLIUM_DRIVER", "__GL_LOG_LEVEL", "LD_DEBUG",
                                       "XDG_SESSION_TYPE", "WAYLAND_DEBUG"};
-            for (const char* var : env_vars) {
-                const char* val = getenv(var);
+            for (const char *var: env_vars) {
+                const char *val = getenv(var);
                 std::cerr << var << "=" << (val ? val : "") << std::endl;
             }
 
             std::cerr << "GLFW version: " << glfwGetVersionString() << std::endl;
             glfwWindowHint(GLFW_OPENGL_DEBUG_CONTEXT, GLFW_TRUE);
 
-            char *v_major = getenv("GLFW_CONTEXT_VERSION_MAJOR");
-            if (v_major) {
-                major_v = std::stoi(v_major );
-            }
-            char *v_minor = getenv("GLFW_CONTEXT_VERSION_MINOR");
-            if (v_minor) {
-                minor_v = std::stoi(v_minor);
-            }
             std::cerr << "GLFW_CONTEXT_VERSION_MAJOR=" << ((major_v == -1) ? "" : v_major) << std::endl;
             std::cerr << "GLFW_CONTEXT_VERSION_MINOR=" << ((minor_v == -1) ? "" : v_minor) << std::endl;
         }
@@ -231,33 +238,67 @@ namespace Manager {
         glfwSetErrorCallback(ErrorCallback);
 
         if (!glfwInit()) {
-            std::cerr << "ERROR: could not initialize GLFW3" << std::endl;
+            std::cerr << "Error: could not initialize GLFW3" << std::endl;
             std::terminate();
         }
-        bool opengl_es_loader = false;
-        glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
+        bool opengl_es_loader;
+
         glfwWindowHint(GLFW_SCALE_TO_MONITOR, GLFW_TRUE);
+        glfwWindowHint(GLFW_SRGB_CAPABLE, GLFW_TRUE);
+        glfwWindowHint(GLFW_RED_BITS, 8);
+        glfwWindowHint(GLFW_GREEN_BITS, 8);
+        glfwWindowHint(GLFW_BLUE_BITS, 8);
+        glfwWindowHint(GLFW_ALPHA_BITS, 8);
+        glfwWindowHint(GLFW_DEPTH_BITS, 0);
+        glfwWindowHint(GLFW_STENCIL_BITS, 0);
+        glfwWindowHint(GLFW_STEREO, GLFW_FALSE);
+        glfwWindowHint(GLFW_SAMPLES, 0);
+
+
+        // Default window setup
 
 #ifndef __APPLE__  // linux, windows, termux
-    #ifdef USE_GL
+#ifdef USE_GL
         // Use OpenGL 4.1 context
         glfwWindowHint(GLFW_CLIENT_API, GLFW_OPENGL_API);
-        glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, (major_v == -1) ? 4 : major_v);
-        glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, (minor_v == -1) ? 1 : major_v);
-    #else
+        major_v = (major_v == -1) ? 4 : major_v;
+        minor_v = (minor_v == -1) ? 1 : minor_v;
+        glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, major_v);
+        glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, minor_v);
+        glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
+        glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);
+        opengl_es_loader = false;
+        if (debug) {
+            std::cerr << "Using OpenGL " << major_v << "." << minor_v << " core profile (forward compatible)\n";
+        }
+#else
         // OpenGL ES 2.0
         glfwWindowHint(GLFW_CLIENT_API, GLFW_OPENGL_ES_API);
         glfwWindowHint(GLFW_CONTEXT_CREATION_API, GLFW_EGL_CONTEXT_API);
-        glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, (major_v == -1) ? 2 : major_v);
-        glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, (minor_v == -1) ? 0 : minor_v);
-
+        major_v = (major_v == -1) ? 2 : major_v;
+        minor_v = (minor_v == -1) ? 0 : minor_v;
+        glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, major_v);
+        glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, minor_v);
         opengl_es_loader = true;
-    #endif
+        if (debug) {
+            std::cerr << "Using OpenGL ES " << major_v << "." << minor_v << " with EGL\n";
+        }
+#endif
 #else
         // Native macOS use OpenGL 4.1
-        glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, (major_v == -1) ? 4 : major_v);
-        glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, (minor_v == -1) ? 1 : major_v);
+        major_v = (major_v == -1) ? 4 : major_v;
+        minor_v = (minor_v == -1) ? 1 : minor_v;
+        glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, major_v);
+        glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, minor_v);
+        glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
+        glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);
+        opengl_es_loader = false;
+        if (debug) {
+            std::cerr << "Using OpenGL " << major_v << "." << minor_v << " core profile (forward compatible)\n";
+        }
 #endif
+
+
         if (debug) {
             std::cerr << "Creating window with size " << width << "x" << height << std::endl;
         }
@@ -266,13 +307,13 @@ namespace Manager {
 
         if (!window) {
             glfwTerminate();
-            std::cerr << "ERROR: glfwCreateWindow failed\n";
+            std::cerr << "Error: glfwCreateWindow failed\n";
             std::exit(-1);
         }
 #if defined(_WIN32) || defined(_WIN64)
         GLFWimage iconimage;
         getWindowIconImage(&iconimage);
-          glfwSetWindowIcon(window, 1, &iconimage);
+        glfwSetWindowIcon(window, 1, &iconimage);
 #endif
 
         // https://stackoverflow.com/questions/7676971/pointing-to-a-function-that-is-a-class-member-glfw-setkeycallback/28660673#28660673
@@ -309,17 +350,18 @@ namespace Manager {
         glfwSetWindowSizeCallback(window, func_resize);
 
         if (!window) {
-            std::cerr << "ERROR: could not create window with GLFW3" << std::endl;
+            std::cerr << "Error: could not create window with GLFW3" << std::endl;
             glfwTerminate();
             std::exit(-1);
         }
         glfwMakeContextCurrent(window);
-
+#ifndef BUILDING_LIBGW
         // Use glad loader
         if (opengl_es_loader) {
             if (!gladLoadGLES2Loader((GLADloadproc) glfwGetProcAddress)) {
                 std::cerr << "Error: failed to initialize GLAD ES\n";
             }
+
             if (debug) {
                 if (GLAD_GL_ES_VERSION_2_0) {
                     std::cerr << "OpenGL ES 2.0 is supported\n";
@@ -327,10 +369,12 @@ namespace Manager {
                     std::cerr << "OpenGL ES 2.0 is not supported\n";
                 }
             }
+
         } else {
             if (!gladLoadGLLoader((GLADloadproc) glfwGetProcAddress)) {
                 std::cerr << "Error: failed to initialize GLAD GL\n";
             }
+
             if (debug) {
                 if (GLAD_GL_VERSION_4_1) {
                     std::cerr << "OpenGL 4.1 is supported\n";
@@ -340,14 +384,20 @@ namespace Manager {
                     std::cerr << "OpenGL 2.1 is " << ((GLAD_GL_VERSION_2_1) ? "" : "not ") << "supported\n";
                 }
             }
+
         }
+#else
+        if (opengl_es_loader) {}  // suppress unused compiler warning
+#endif
         if (debug) {
+#ifndef BUILDING_LIBGW
             const GLubyte *rend = glGetString(GL_RENDERER);
             const GLubyte *ver = glGetString(GL_VERSION);
             const GLubyte *ven = glGetString(GL_VENDOR);
             std::cerr << "OpenGL renderer: " << rend << std::endl;
             std::cerr << "OpenGL version: " << ver << std::endl;
             std::cerr << "OpenGL vendor: " << ven << std::endl;
+#endif
             float xscale;
             float yscale;
             int windX, windY;
@@ -367,18 +417,18 @@ namespace Manager {
 
     void GwPlot::initBack(int width, int height) {
         if (!glfwInit()) {
-            std::cerr << "ERROR: could not initialize GLFW3" << std::endl;
+            std::cerr << "Error: could not initialize GLFW3" << std::endl;
             std::exit(-1);
         }
 
         glfwWindowHint(GLFW_VISIBLE, GLFW_FALSE);
-        backWindow = glfwCreateWindow(width, height, "GW", NULL, NULL);
-        if (!backWindow) {
-            std::cerr << "ERROR: could not create back window with GLFW3" << std::endl;
+        window = glfwCreateWindow(width, height, "GW", NULL, NULL);
+        if (!window) {
+            std::cerr << "Error: could not create back window with GLFW3" << std::endl;
             glfwTerminate();
             std::terminate();
         }
-        glfwMakeContextCurrent(backWindow);
+        glfwMakeContextCurrent(window);
         drawToBackWindow = true;
 
         setGlfwFrameBufferSize();
@@ -414,6 +464,7 @@ namespace Manager {
         hts_set_fai_filename(f, reference.c_str());
         hts_set_threads(f, opts.threads);
         bams.push_back(f);
+        bam_paths.push_back(bam_path);
         sam_hdr_t *hdr_ptr = sam_hdr_read(f);
         headers.push_back(hdr_ptr);
         hts_idx_t *idx = sam_index_load(f, bam_path.c_str());
@@ -808,6 +859,46 @@ namespace Manager {
 
                 sContext->abandonContext();
                 sk_sp<const GrGLInterface> interface = GrGLMakeNativeInterface();
+#ifndef OLD_SKIA
+                sContext = GrDirectContexts::MakeGL(interface).release();
+
+                GrGLFramebufferInfo framebufferInfo;
+                framebufferInfo.fFBOID = 0;
+                framebufferInfo.fFormat = GL_RGBA8;
+
+                auto backendRenderTarget = GrBackendRenderTargets::MakeGL(
+                        fb_width,
+                        fb_height,
+                        0,        // sampleCnt
+                        0,        // stencilBits
+                        framebufferInfo
+                );
+
+                if (!backendRenderTarget.isValid()) {
+                    std::cerr << "Error: backendRenderTarget was invalid" << std::endl;
+                    glfwTerminate();
+                    std::exit(-1);
+                }
+
+                SkSurfaceProps props(0, kUnknown_SkPixelGeometry);
+                sSurface = SkSurfaces::WrapBackendRenderTarget(
+                        sContext,
+                        backendRenderTarget,
+                        kBottomLeft_GrSurfaceOrigin,
+                        kRGBA_8888_SkColorType,
+                        nullptr,
+                        &props
+                ).release();
+
+                if (!sSurface) {
+                    std::cerr
+                            << "Error: sSurface could not be initialized (nullptr). The frame buffer format needs changing\n";
+                    std::exit(-1);
+                }
+
+                auto imageInfo = SkImageInfo::MakeN32Premul(fb_width, fb_height);
+                rasterSurface = SkSurfaces::Raster(imageInfo);
+#else
                 sContext = GrDirectContext::MakeGL(interface).release();
 
                 GrGLFramebufferInfo framebufferInfo;
@@ -834,6 +925,7 @@ namespace Manager {
                 }
 
                 rasterSurface = SkSurface::MakeRasterN32Premul(fb_width, fb_height);
+#endif
                 rasterCanvas = rasterSurface->getCanvas();
                 rasterSurfacePtr = &rasterSurface;
 
@@ -990,7 +1082,7 @@ namespace Manager {
             }
         } else {
             monitorScale = 1;
-            glfwGetFramebufferSize(backWindow, &fb_width, &fb_height);
+//            glfwGetFramebufferSize(window, &fb_width, &fb_height);
         }
         gap = monitorScale * 10;
         fonts.setOverlayHeight(monitorScale);
@@ -1047,6 +1139,11 @@ namespace Manager {
             } else {
                 if (yScaling > 3*monitorScale) {
                     pH = yScaling - monitorScale;
+                    //                     if (monitorScale > 1) {
+                    //                        pH = yScaling - monitorScale;
+                    //                    } else {
+                    //                        pH = yScaling - 2;
+                    //                    }
                 } else {
                     pH = yScaling;
                 }
@@ -1102,6 +1199,7 @@ namespace Manager {
     void GwPlot::drawScreen(SkCanvas* canvas, GrDirectContext* sContext, SkSurface *sSurface) {
 
 //        std::chrono::high_resolution_clock::time_point initial = std::chrono::high_resolution_clock::now();
+
         SkCanvas *canvasR = rasterCanvas;
 
         canvas->drawPaint(opts.theme.bgPaint);
@@ -1120,6 +1218,8 @@ namespace Manager {
             setScaling();
 
             SkRect clip;
+
+            // Draw background image
             if (!imageCacheQueue.empty() && collections.size() > 1) {
                 canvasR->drawImage(imageCacheQueue.back().second, 0, 0);
                 clip.setXYWH(0, 0, fb_width, refSpace);
@@ -1129,6 +1229,7 @@ namespace Manager {
                 canvasR->restore();
             }
 
+            // Update changes parts of image
             for (auto &cl: collections) {
                 if (cl.skipDrawingCoverage && cl.skipDrawingReads) {  // keep read and coverage area
                     continue;
@@ -1576,7 +1677,11 @@ namespace Manager {
                                           data = SkData::MakeFromFileName(fname);
                                           if (!data)
                                               throw std::runtime_error("Error: file not found");
+#ifndef OLD_SKIA
+                                          sk_sp<SkImage> image = SkImages::DeferredFromEncodedData(data);
+#else
                                           sk_sp<SkImage> image = SkImage::MakeFromEncoded(data);
+#endif
                                           if (!image)
                                               throw std::runtime_error("Failed to decode an image");
                                           // if image is not explicitly decoded, it might be lazily decoded during drawing
@@ -1666,9 +1771,7 @@ namespace Manager {
         fetchRefSeqs();
         processBam();
         setScaling();
-        if (yScaling == 0) {
-            return;
-        }
+
         canvas->drawPaint(opts.theme.bgPaint);
         SkRect clip;
 
@@ -1840,7 +1943,14 @@ namespace Manager {
     void GwPlot::rasterToPng(const char* path) {
         sk_sp<SkImage> img(rasterSurfacePtr[0]->makeImageSnapshot());
         if (!img) { return; }
+
+#ifndef OLD_SKIA
+        SkPngEncoder::Options options;
+        sk_sp<SkData> png(SkPngEncoder::Encode(nullptr, img.get(), options));
+#else
         sk_sp<SkData> png(img->encodeToData());
+#endif
+
         if (!png) { return; }
         FILE* fout = fopen(path, "w");
         fwrite(png->data(), 1, png->size(), fout);
@@ -1849,7 +1959,12 @@ namespace Manager {
 
     void imageToPng(sk_sp<SkImage> &img, std::filesystem::path &path) {
         if (!img) { return; }
+#ifndef OLD_SKIA
+        SkPngEncoder::Options options;
+        sk_sp<SkData> png(SkPngEncoder::Encode(nullptr, img.get(), options));
+#else
         sk_sp<SkData> png(img->encodeToData());
+#endif
         if (!png) { return; }
 #if defined(_WIN32)
 	const wchar_t* outp = path.c_str();
@@ -1864,7 +1979,12 @@ namespace Manager {
 
     void imagePngToStdOut(sk_sp<SkImage> &img) {
         if (!img) { return; }
+#ifndef OLD_SKIA
+        SkPngEncoder::Options options;
+        sk_sp<SkData> png(SkPngEncoder::Encode(nullptr, img.get(), options));
+#else
         sk_sp<SkData> png(img->encodeToData());
+#endif
         if (!png) { return; }
         FILE* fout = stdout;
 #if defined(_WIN32) || defined(_WIN64)
@@ -1883,7 +2003,12 @@ namespace Manager {
 
     void imagePngToFile(sk_sp<SkImage> &img, std::string path) {
         if (!img) { return; }
+#ifndef OLD_SKIA
+        SkPngEncoder::Options options;
+        sk_sp<SkData> png(SkPngEncoder::Encode(nullptr, img.get(), options));
+#else
         sk_sp<SkData> png(img->encodeToData());
+#endif
         if (!png) { return; }
         FILE* fout = fopen(path.c_str(), "w");
         fwrite(png->data(), 1, png->size(), fout);
