@@ -21,9 +21,15 @@
 
 #define SK_GL
 
-#include "include/gpu/GrBackendSurface.h"
-#include "include/gpu/GrDirectContext.h"
-#include "include/gpu/gl/GrGLInterface.h"
+#ifndef OLD_SKIA
+    #include "include/gpu/ganesh/GrBackendSurface.h"
+    #include "include/gpu/ganesh/GrDirectContext.h"
+    #include "include/gpu/ganesh/gl/GrGLInterface.h"
+#else
+    #include "include/gpu/GrBackendSurface.h"
+    #include "include/gpu/GrDirectContext.h"
+    #include "include/gpu/gl/GrGLInterface.h"
+#endif
 #include "include/core/SkCanvas.h"
 #include "include/core/SkColorSpace.h"
 #include "include/core/SkSurface.h"
